// LBandClass.cpp : Implementation of CLBandClass
#include "stdafx.h"
#include "IEditor.h"
#include "LBandClass.h"
#include <ExDispID.h>  // For IDispatch::Invoke()
#include <process.h>

#include "include/Platform.h"
#include "include/Scintilla.h"
#include "include/SciLexer.h"

/////////////////////////////////////////////////////////////////////////////
// CLBandClass

static OPENFILENAME m_ofn;
static ULONG        m_nSearchAnchor = 0;
static HWND         m_hwndFindTarget = 0;

/////////////////////////////////////////////////////////////////////////////
// IOleWindow Methods

STDMETHODIMP CLBandClass::GetWindow(HWND* phwnd)
{
	*phwnd = m_hWnd;
	return S_OK;
}

STDMETHODIMP CLBandClass::ContextSensitiveHelp(BOOL)
{
	return E_NOTIMPL;
}

/////////////////////////////////////////////////////////////////////////////
//
// IObjectWithSite Methods
//
STDMETHODIMP CLBandClass::SetSite(IUnknown* pUnkSite)
{
	//
	// If punkSite is not NULL, a new site is being set.
	//
	if (pUnkSite)
	{
		//
		// If a site is being held, release it and
		// release the in-place object and frame interfaces.
		//
		if (m_pSite)
		{
			m_pSite->Release();
			m_pSite = NULL;
		}

		// Get the parent window.
		IOleWindow* pOleWindow;
		if (SUCCEEDED(pUnkSite->QueryInterface(IID_IOleWindow, (LPVOID*)&pOleWindow)))
		{
			pOleWindow->GetWindow(&m_hwndParent);
			pOleWindow->Release();
		}

		_ASSERT(m_hwndParent);
		if (!m_hwndParent) { return E_FAIL; }

		RECT rc;
		::GetClientRect(m_hwndParent, &rc);
		Create(m_hwndParent, rc, _T("IEditorBarSuperClass"));

		// Get and keep the IInputObjectSite pointer.
		HRESULT hr = pUnkSite->QueryInterface(IID_IInputObjectSite, (LPVOID*)&m_pSite);
		_ASSERT(SUCCEEDED(hr));

		//
		// Get the IWebBrowser2 interface of Internet Explorer
		// This is so we can automate the browser.
		//
		IOleCommandTarget* pCmdTarget;
		hr = pUnkSite->QueryInterface(IID_IOleCommandTarget, (LPVOID*)&pCmdTarget);
		if (SUCCEEDED(hr))
		{
			IServiceProvider* pSP;
			hr = pCmdTarget->QueryInterface(IID_IServiceProvider, (LPVOID*)&pSP);

			pCmdTarget->Release();

			if (SUCCEEDED(hr))
			{
				if (m_pBrowser)
				{
					m_pBrowser->Release();
					m_pBrowser = NULL;
				}
				
				hr = pSP->QueryService(SID_SWebBrowserApp, IID_IWebBrowser2, (LPVOID*)&m_pBrowser);

				pSP->Release();
				
				if (SUCCEEDED(hr))
				{
					// Advise the IWebBrowser2 in the main window that
					// we will be listening to its events.
					//
					// m_pBrowser - pointer to the IWebBrowser2 in the main
					//             window
					//
					// AtlAdvise(m_pBrowser, GetUnknown(), DIID_DWebBrowserEvents2, &m_dwCookie);
				}
			}
		}

	}

	return S_OK;
}

STDMETHODIMP CLBandClass::GetSite(REFIID riid, void **ppvSite)
{
	*ppvSite = NULL;
	if(m_pSite)
	{
		return m_pSite->QueryInterface(riid,ppvSite);
	}

	return E_FAIL;
}


/////////////////////////////////////////////////////////////////////////////
//
// IDockingWindow, IDeskBand Methods
//

STDMETHODIMP CLBandClass::ShowDW(BOOL fShow)
{
	if(m_hWnd)
	{
		if(fShow)
		{
			::ShowWindow(m_hWnd,SW_SHOW);
		}
		else
		{
			::ShowWindow(m_hWnd,SW_HIDE);
		}
	}
	return S_OK;
}


STDMETHODIMP CLBandClass::CloseDW(unsigned long dwReserved)
{
	ShowDW(FALSE);
	if(::IsWindow(m_hWnd))
		::DestroyWindow(m_hWnd);

	m_hWnd = NULL;
	return S_OK;
}


STDMETHODIMP CLBandClass::ResizeBorderDW(const RECT*  prcBorder,
                                 IUnknown* punkToolbarSite,
                                 BOOL      fReserved)
{
	return E_NOTIMPL;
}

STDMETHODIMP CLBandClass::GetBandInfo(DWORD dwBandID, DWORD dwViewMode, DESKBANDINFO* pdbi)
{
	if(pdbi)
	{
		if(pdbi->dwMask & DBIM_MINSIZE)
		{
			pdbi->ptMinSize.x = 200;
			pdbi->ptMinSize.y = 200;
		}

		if(pdbi->dwMask & DBIM_MAXSIZE)
		{
			pdbi->ptMaxSize.x = -1;
			pdbi->ptMaxSize.y = -1;
		}

		if(pdbi->dwMask & DBIM_INTEGRAL)
		{
			pdbi->ptIntegral.x = 1;
			pdbi->ptIntegral.y = 1;
		}

		if(pdbi->dwMask & DBIM_ACTUAL)
		{
			pdbi->ptActual.x = 0;
			pdbi->ptActual.y = 0;
		}

		if(pdbi->dwMask & DBIM_TITLE)
		{
			lstrcpyW(pdbi->wszTitle, L"IEditor");
		}

		if(pdbi->dwMask & DBIM_MODEFLAGS)
		{
			pdbi->dwModeFlags = DBIMF_VARIABLEHEIGHT;

		}

		if(pdbi->dwMask & DBIM_BKCOLOR)
		{
			pdbi->dwMask &= ~DBIM_BKCOLOR;
		}
		
		return S_OK;
	}

	return E_INVALIDARG;
}

LRESULT CLBandClass::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	long lStyle = 0;
	
	lStyle = ::GetWindowLong(m_hWnd, GWL_STYLE);
	HMENU hMenu;

	lStyle &= WS_DISABLED;

	::SetWindowLong(m_hWnd, GWL_STYLE, lStyle);

	hMenu = ::LoadMenu(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDR_MAINFRAME));
	::SetMenu(m_hWnd, hMenu);

	m_hScintilla = ::LoadLibrary(_T("SciLexer.DLL"));
		
	if(m_hScintilla)
	{
		m_hwndScintilla = CreateWindowEx(WS_EX_CLIENTEDGE, _T("Scintilla"),_T(""), WS_CHILD|WS_VISIBLE|WS_TABSTOP,0,0,100,100, m_hWnd, NULL, _Module.GetModuleInstance(), NULL);
		if(m_hwndScintilla)
		{
 			FileControlInitialize(m_hwndScintilla);

			m_hwndFindTarget = m_hwndScintilla;

			m_crBackDefault = RGB(235, 235, 235);
			m_crBackUnknown = RGB(255, 255, 0);
			m_crTextDefault = RGB(0, 0, 50);
			m_crTextSpecial = RGB(0, 0, 150);
			m_crTextOther = RGB(150, 0, 150);
			m_crTextComments = RGB(0, 150, 0);
			m_crTextUnknown = RGB(255, 0, 0);
			m_crTextTag = RGB(0,0,255);
			
			// printing related
			m_hDevMode = NULL;
			m_hDevNames = NULL;
			
			// fonts
			m_bKeywordBold = FALSE;
			m_nDefaultFontSize = 10;
			m_nHtmlFontSize = 10;
			_tcscpy(m_strFontHtml,_T("Courier New"));
			_tcscpy(m_strFontDefault,_T("Courier New"));

			ReadSettings();

			SendEditor(SCI_SETLEXER, SCLEX_NULL, NULL);
			SetAStyle(STYLE_DEFAULT, m_crTextDefault, m_crBackDefault, m_nDefaultFontSize, m_strFontDefault);
			SendEditor(SCI_STYLECLEARALL,NULL,NULL);	// Copies global style to all others
			SendEditor(SCI_SETWRAPMODE, SC_WRAP_WORD,NULL);
			SendEditor(SCI_SETPRINTCOLOURMODE, SC_PRINT_COLOURONWHITE, 0);

			// html colorise
			SendEditor(SCI_SETLEXER, SCLEX_HTML, NULL);
			SetAStyle(SCE_H_DEFAULT, m_crTextDefault, m_crBackDefault, m_nHtmlFontSize, m_strFontHtml);
			SendEditor(SCI_STYLECLEARALL,NULL,NULL);	// Copies global style to all others

			SetAStyle(SCE_H_TAG, m_crTextTag, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_TAGEND, m_crTextTag, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_QUESTION, m_crTextUnknown, m_crBackUnknown,NULL,NULL);
			SetAStyle(SCE_H_ATTRIBUTE, m_crTextTag, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_ATTRIBUTEUNKNOWN, m_crTextSpecial, m_crBackDefault,NULL,NULL); //<qth:4:X
			SetAStyle(SCE_H_NUMBER, m_crTextComments, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_OTHER, m_crTextOther, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_ENTITY, m_crTextOther, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_COMMENT, m_crTextComments, m_crBackDefault,NULL,NULL);

			SendEditor(SCI_STYLESETBOLD,SCE_H_TAG,m_bKeywordBold);

			SetAStyle(SCE_H_DOUBLESTRING, m_crTextComments, m_crBackDefault,NULL,NULL);
			SetAStyle(SCE_H_SINGLESTRING, m_crTextComments, m_crBackDefault,NULL,NULL);

			SendEditor(SCI_SETSTYLEBITS, 6, 0);
			SendEditor(SCI_SETKEYWORDS, 0, reinterpret_cast<LPARAM>(szElements));
			SendEditor(SCI_SETKEYWORDS, 1, reinterpret_cast<LPARAM>(szAttributes));
			
			SendEditor(SCI_COLOURISE, 0, -1);
		}
	}

	return 1;
}

LRESULT CLBandClass::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int nWidth = LOWORD(lParam);
	int nHeight = HIWORD(lParam);
	if(m_hwndScintilla)	::MoveWindow(m_hwndScintilla,0,0,nWidth,nHeight,FALSE);

	return 0;
}

void CLBandClass::FileControlInitialize(HWND hwnd)
{
     static TCHAR szFilter[] = _T("All Files (*.*)\0*.*\0\0") \
                               _T("HTML Files (*.htm)\0*.htm\0") \
                               _T("Text Files (*.txt)\0*.txt\0") ;
     
     m_ofn.lStructSize       = sizeof(OPENFILENAME);
     m_ofn.hwndOwner         = hwnd;
     m_ofn.hInstance         = NULL;
     m_ofn.lpstrFilter       = szFilter;
     m_ofn.lpstrCustomFilter = NULL;
     m_ofn.nMaxCustFilter    = 0;
     m_ofn.nFilterIndex      = 0;
     m_ofn.lpstrFile         = NULL;          // Set in Open and Close functions
     m_ofn.nMaxFile          = MAX_PATH;
     m_ofn.lpstrFileTitle    = NULL;          // Set in Open and Close functions
     m_ofn.nMaxFileTitle     = MAX_PATH;
     m_ofn.lpstrInitialDir   = NULL;
     m_ofn.lpstrTitle        = NULL;
     m_ofn.Flags             = 0;             // Set in Open and Close functions
     m_ofn.nFileOffset       = 0;
     m_ofn.nFileExtension    = 0;
     m_ofn.lpstrDefExt       = TEXT ("txt");
     m_ofn.lCustData         = 0L;
     m_ofn.lpfnHook          = NULL;
     m_ofn.lpTemplateName    = NULL;
}

BOOL CLBandClass::FileOpenDlg(HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName)
{
     m_ofn.hwndOwner         = hwnd;
     m_ofn.lpstrFile         = pstrFileName;
     m_ofn.lpstrFileTitle    = pstrTitleName;
     m_ofn.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT;
     
     return GetOpenFileName(&m_ofn);
}

BOOL CLBandClass::FileSaveDlg(HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName)
{
     m_ofn.hwndOwner         = hwnd;
     m_ofn.lpstrFile         = pstrFileName;
     m_ofn.lpstrFileTitle    = pstrTitleName;
     m_ofn.Flags             = OFN_OVERWRITEPROMPT;
     
     return GetSaveFileName(&m_ofn);
}

BOOL CLBandClass::FileRead(PTSTR pstrFileName)
{
     BYTE   bySwap;
     DWORD  dwBytesRead;
     HANDLE hFile;
     int    i = 0;
	 int    nFileLength = 0;
	 int    nUniTest = 0;
     PBYTE  pBuffer, pText, pConv;

     // Open the file.

     if(INVALID_HANDLE_VALUE == (hFile = CreateFile (pstrFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL))) return FALSE ;

     // Get file size in bytes and allocate memory for read.
     // Add an extra two bytes for zero termination.
                    
     nFileLength = ::GetFileSize(hFile, NULL);
     pBuffer = (BYTE*)malloc(nFileLength + 2);

     // Read file and put terminating zeros at end.
     
     ::ReadFile(hFile, pBuffer, nFileLength, &dwBytesRead, NULL);
     ::CloseHandle (hFile);
     pBuffer[nFileLength] = '\0' ;
     pBuffer[nFileLength + 1] = '\0' ;

     // Test to see if the text is Unicode

     nUniTest = IS_TEXT_UNICODE_SIGNATURE | IS_TEXT_UNICODE_REVERSE_SIGNATURE ;
     
     if(IsTextUnicode (pBuffer, nFileLength, &nUniTest))
     {
          pText = pBuffer + 2 ;
          nFileLength -= 2 ;

          if(nUniTest & IS_TEXT_UNICODE_REVERSE_SIGNATURE)
          {
               for(i=0; i<nFileLength/2; i++)
               {
                    bySwap = ((BYTE*) pText)[2*i];
                    ((BYTE *) pText) [2*i] = ((BYTE *) pText)[2*i+1];
                    ((BYTE *) pText) [2*i + 1] = bySwap;
               }
          }

          // Allocate memory for possibly converted string

          pConv = (BYTE*)malloc(nFileLength + 2);

          // If the edit control is not Unicode, convert Unicode text to 
          // non-Unicode (ie, in general, wide character).

#ifndef UNICODE
          WideCharToMultiByte(CP_ACP, 0, (PWSTR) pText, -1, (char*)pConv, nFileLength + 2, NULL, NULL);
         // If the edit control is Unicode, just copy the string
#else
          _tcscpy((PTSTR)pConv, (PTSTR)pText);
#endif
     }
     else // the file is not Unicode
     {
          pText = pBuffer;

          // Allocate memory for possibly converted string.

          pConv = (BYTE*)malloc(2*nFileLength+2);

          // If the edit control is Unicode, convert ASCII text.

#ifdef UNICODE
          MultiByteToWideChar (CP_ACP, 0, (const char*)pText, -1, (PTSTR) pConv, nFileLength + 1) ;
          // If not, just copy buffer
#else
          _tcscpy((PTSTR)pConv, (PTSTR)pText);
#endif
     }
	SendEditor(SCI_CLEARALL, 0, 0);
	SendEditor(SCI_ADDTEXT,(WPARAM)nFileLength,reinterpret_cast<LPARAM>(pConv));
	SendEditor(SCI_EMPTYUNDOBUFFER, 0, 0);

    free (pBuffer);
    free (pConv);
   
    return TRUE;
}

BOOL CLBandClass::FileWrite(PTSTR pstrFileName)
{
     DWORD  dwBytesWritten = 0;
     HANDLE hFile = NULL;
     int    nLength  = 0;
     PTSTR  pstrBuffer;
     WORD   wByteOrderMark = 0xFEFF;

     // Open the file, creating it if necessary
     
     if(INVALID_HANDLE_VALUE == (hFile = CreateFile (pstrFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL))) return FALSE;

     // Get the number of characters in the edit control and allocate
     // memory for them.
     
     nLength = SendEditor(SCI_GETTEXTLENGTH,0,0);
     pstrBuffer = (PTSTR)malloc((nLength + 1)*sizeof(TCHAR));
     
     if(!pstrBuffer)
     {
          CloseHandle(hFile);
          return FALSE;
     }

     // If the edit control will return Unicode text, write the
     // byte order mark to the file.

#ifdef UNICODE
     WriteFile(hFile, &wByteOrderMark, 2, &dwBytesWritten, NULL);
#endif

     // Get the edit buffer and write that out to the file.
     
	 SendEditor(SCI_GETTEXT,nLength + 1,(long)pstrBuffer);
     WriteFile(hFile, pstrBuffer, nLength*sizeof(TCHAR), &dwBytesWritten, NULL);
     
     if((nLength*sizeof(TCHAR))!=(int)dwBytesWritten)
     {
          CloseHandle(hFile);
          free(pstrBuffer);
          return FALSE;
     }
     
     CloseHandle(hFile);
     free(pstrBuffer);
     
     return TRUE;
}

BOOL CLBandClass::ReadSettings()
{
    TCHAR lpFileName[MAX_PATH] = {0};
	GetModuleFileName(_Module.GetModuleInstance(), lpFileName, MAX_PATH);

	for(int i=MAX_PATH;i>0;i--)
	{
		if(lpFileName[i]=='\\')
		{
			lpFileName[i+1]='\0';
			break;
		}
	}
	_tcscat(lpFileName,_T("SETTINGS.XML"));

	HANDLE hFile;
	int nFileLength = 0;
    TCHAR* lpBuffer = 0;
	DWORD dwBytesRead = 0;

	if(INVALID_HANDLE_VALUE == (hFile = CreateFile (lpFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL))) return FALSE ;

	// Get file size in bytes and allocate memory for read.
	// Add an extra two bytes for zero termination.
                    
	nFileLength = ::GetFileSize(hFile, NULL);
	lpBuffer = (TCHAR*)malloc(nFileLength + 2);

	// Read file and put terminating zeros at end.

	::ReadFile(hFile, lpBuffer, nFileLength, &dwBytesRead, NULL);
	::CloseHandle(hFile);
	lpBuffer[nFileLength] = '\0';
	lpBuffer[nFileLength + 1] = '\0';

	BOOL bTag = FALSE;
	BOOL bValue = FALSE;
	TCHAR* lpTag = new TCHAR[nFileLength];
	memset(lpTag,0,nFileLength);
	TCHAR* lpValue = new TCHAR[nFileLength];
	memset(lpValue,0,nFileLength);
	int k = 0;
	int m = 0;

	for(i=0;i<nFileLength-2;i++)
	{
		if(lpBuffer[i]=='<')
		{
			if(lpBuffer[i+1]!='/')
			{
				k = 0;
				bTag = TRUE;
				memset(lpTag,0,nFileLength);
			}
			else
			{
				if(_tcslen(lpTag)>0)
				{
					if(_tcsstr(lpTag,_T("ColorBackDefault")))       m_crBackDefault = _ttol(lpValue);  //RGB(235, 235, 235);
					else if(_tcsstr(lpTag,_T("ColorBackUnknown")))  m_crBackUnknown = _ttol(lpValue);  //RGB(255, 255, 0);
					else if(_tcsstr(lpTag,_T("ColorTextDefault")))  m_crTextDefault = _ttol(lpValue);  //RGB(0, 0, 50);
					else if(_tcsstr(lpTag,_T("ColorTextSpecial")))  m_crTextSpecial = _ttol(lpValue);  //RGB(0, 0, 150);
					else if(_tcsstr(lpTag,_T("ColorTextOther")))    m_crTextOther = _ttol(lpValue);    //RGB(150, 0, 150);
					else if(_tcsstr(lpTag,_T("ColorTextComments"))) m_crTextComments = _ttol(lpValue); //RGB(0, 150, 0);
					else if(_tcsstr(lpTag,_T("ColorTextUnknown")))  m_crTextUnknown = _ttol(lpValue);  //RGB(255, 0, 0);
					else if(_tcsstr(lpTag,_T("ColorTextTag")))      m_crTextTag = _ttol(lpValue);      //RGB(0,0,255);
					else if(_tcsstr(lpTag,_T("KeywordBold")))       m_bKeywordBold = (BOOL)_ttol(lpValue);
					else if(_tcsstr(lpTag,_T("DefaultFontSize")))   m_nDefaultFontSize = _ttol(lpValue);
					else if(_tcsstr(lpTag,_T("HtmlFontSize")))      m_nHtmlFontSize = _ttol(lpValue);
					else if(_tcsstr(lpTag,_T("DefaultFontName")))   _tcscpy(m_strFontHtml, lpValue);
					else if(_tcsstr(lpTag,_T("HtmlFontName")))      _tcscpy(m_strFontDefault, lpValue);
				}
				memset(lpTag,0,nFileLength);
			}
			
			bValue = FALSE;
		}
		else if(lpBuffer[i]=='>')
		{
			if(bTag)
			{
				m = 0;
				bValue = TRUE;
				memset(lpValue,0,nFileLength);
			}
			bTag = FALSE;
		}
		
		if(bTag)
		{
			if(lpBuffer[i]=='<' || lpBuffer[i]=='\r' || lpBuffer[i]=='\n' || lpBuffer[i]==' ') continue;
			lpTag[k]=lpBuffer[i];
			k++;
		}

		if(bValue)
		{
			if(lpBuffer[i]=='<' || lpBuffer[i]=='>' || lpBuffer[i]=='\r' || lpBuffer[i]=='\n') continue;
			lpValue[m]=lpBuffer[i];
			m++;
		}
	}

	delete [] lpBuffer;
	delete [] lpTag;
	delete [] lpValue;

	return TRUE;
}

void CLBandClass::PageSource()
{
	HRESULT hRes = S_OK;

	CComPtr<IDispatch>        pDocDisp;
	CComQIPtr<IHTMLDocument2> pHtmlDoc2;

	if(m_pBrowser)
	{
		hRes = m_pBrowser->get_Document(&pDocDisp);
		if(SUCCEEDED(hRes) && pDocDisp)
		{
			hRes = pDocDisp->QueryInterface(IID_IHTMLDocument2, (void**)&pHtmlDoc2);
			if(SUCCEEDED(hRes) && pHtmlDoc2)
			{
				BOOL bFindDiv = FALSE;

				CComQIPtr<IHTMLElementCollection> pCollection;
				hRes = pHtmlDoc2->get_all(&pCollection);

				if(SUCCEEDED(hRes))
				{
					CComVariant vIndex;
					vIndex.vt = VT_I4;
					
					long nItems = 0;
					long i = 0;
					
					hRes = pCollection->get_length(&nItems);
					if(SUCCEEDED(hRes))
					{
						CComPtr<IDispatch> pHtml;
						for(i=0;i<nItems;i++)
						{
							vIndex.lVal = i;
							hRes = pCollection->item(vIndex, vIndex, &pHtml);
							if(SUCCEEDED(hRes) && pHtml)
							{
								CComQIPtr<IHTMLElement> pElement;
								hRes = pHtml->QueryInterface(IID_IHTMLElement, (void**)&pElement);
								if(SUCCEEDED(hRes) && pElement)
								{
									CComBSTR bstrTag;
									hRes = pElement->get_tagName(&bstrTag);
									bstrTag.ToUpper();
									if(SUCCEEDED(hRes) && bstrTag == "HTML")
									{
										CComBSTR bstrHtml;
										hRes = pElement->get_outerHTML(&bstrHtml);
										if(SUCCEEDED(hRes) && bstrHtml)
										{
											CString strHtml = CString(bstrHtml);
											SendEditor(SCI_CLEARALL, 0, 0);
											SendEditor(SCI_ADDTEXT,(WPARAM)(strHtml.GetLength()*sizeof(TCHAR)),reinterpret_cast<LPARAM>(strHtml.GetBuffer(strHtml.GetLength())));
											SendEditor(SCI_EMPTYUNDOBUFFER, 0, 0);

											strHtml.ReleaseBuffer();
											break;
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	::SetFocus(m_hwndScintilla);
}

//////////////////////////////////////////////////////////////////////////////////////////////
LRESULT CLBandClass::SendEditor(UINT Msg, WPARAM wParam, LPARAM lParam)
{
	return ::SendMessage(m_hwndScintilla, Msg, wParam, lParam);
}
	
void CLBandClass::SetAStyle(int style, COLORREF fore, COLORREF back, int size, const TCHAR* face)
{
	SendEditor(SCI_STYLESETFORE, style, fore);
	SendEditor(SCI_STYLESETBACK, style, back);
	if (size >= 1)	SendEditor(SCI_STYLESETSIZE, style, size);
	if (face) SendEditor(SCI_STYLESETFONT, style, reinterpret_cast<LPARAM>(face));
}

LRESULT CLBandClass::OnFileNew(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SendEditor(SCI_CLEARALL, 0, 0);
	SendEditor(SCI_EMPTYUNDOBUFFER, 0, 0);
	::SetFocus(m_hwndScintilla);
	return 0;
}

LRESULT CLBandClass::OnFileOpen(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	if(FileOpenDlg(m_hwndScintilla, m_strFileName, m_strTitleName))
		FileRead(m_strFileName);

	::SetFocus(m_hwndScintilla);
	
	return 0;
}

LRESULT CLBandClass::OnFileSave(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	if(m_strFileName[0] || SendEditor(SCI_GETTEXTLENGTH,0,0)>0)
	{
		if(FileWrite(m_strFileName)) return 1;
		else	
		{
			::MessageBox(m_hWnd,_T("Could not write the file."),_T("IEditor"),MB_OK|MB_ICONERROR);
			return 0;
		}
	}
	else OnFileSaveAs(wNotifyCode, wID, hWndCtl, bHandled);
	
	::SetFocus(m_hwndScintilla);
	return 0;
}

LRESULT CLBandClass::OnFileSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	if(FileSaveDlg(m_hwndScintilla, m_strFileName, m_strTitleName))
	{
		if(FileWrite(m_strFileName)) return 1;
		else
		{
			::MessageBox(m_hWnd,_T("Could not write the file."),_T("IEditor"),MB_OK|MB_ICONERROR);
			return 0;
		}
	}
	
	::SetFocus(m_hwndScintilla);
	return 0;
}

LRESULT CLBandClass::OnFilePrint(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	PRINTDLG pDlg = {sizeof(PRINTDLG), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	pDlg.hwndOwner = m_hwndScintilla;
	pDlg.hInstance = m_hScintilla;
	pDlg.Flags = PD_USEDEVMODECOPIES | PD_ALLPAGES | PD_RETURNDC;
	pDlg.nFromPage = 1;
	pDlg.nToPage = 1;
	pDlg.nMinPage = 1;
	pDlg.nMaxPage = 0xffffU; // We do not know how many pages in the
	// document until the printer is selected and the paper size is known.
	pDlg.nCopies = 1;
	pDlg.hDC = 0;
	pDlg.hDevMode = m_hDevMode;
	pDlg.hDevNames = m_hDevNames;

	// See if a range has been selected
	int nStartPos = SendEditor(SCI_GETSELECTIONSTART,0,0);
	int nEndPos =  SendEditor(SCI_GETSELECTIONEND,0,0);

	if(nStartPos == nEndPos) pDlg.Flags |= PD_NOSELECTION;
	else pDlg.Flags |= PD_SELECTION;

	if(!::PrintDlg(&pDlg)) return 1;

	m_hDevMode = pDlg.hDevMode;
	m_hDevNames = pDlg.hDevNames;

	HDC hdc = pDlg.hDC;

	RECT rcMargins;
	RECT rcPhysMargins;
	POINT ptPage;
	POINT ptDpi;

	// Get printer resolution
	ptDpi.x = GetDeviceCaps(hdc, LOGPIXELSX);    // dpi in X direction
	ptDpi.y = GetDeviceCaps(hdc, LOGPIXELSY);    // dpi in Y direction

	// Start by getting the physical page size (in device units).
	ptPage.x = GetDeviceCaps(hdc, PHYSICALWIDTH);   // device units
	ptPage.y = GetDeviceCaps(hdc, PHYSICALHEIGHT);  // device units

	// Get the dimensions of the unprintable
	// part of the page (in device units).
	rcPhysMargins.left = GetDeviceCaps(hdc, PHYSICALOFFSETX);
	rcPhysMargins.top = GetDeviceCaps(hdc, PHYSICALOFFSETY);

	// To get the right and lower unprintable area,
	// we take the entire width and height of the paper and
	// subtract everything else.
	rcPhysMargins.right = ptPage.x						// total paper width
	                        - GetDeviceCaps(hdc, HORZRES) // printable width
	                        - rcPhysMargins.left;				// left unprintable margin

	rcPhysMargins.bottom = ptPage.y						// total paper height
	                         - GetDeviceCaps(hdc, VERTRES)	// printable height
	                         - rcPhysMargins.top;				// right unprintable margin

	// At this point, rcPhysMargins contains the widths of the
	// unprintable regions on all four sides of the page in device units.

	rcMargins.left	= rcPhysMargins.left;
	rcMargins.top	= rcPhysMargins.top;
	rcMargins.right	= rcPhysMargins.right;
	rcMargins.bottom = rcPhysMargins.bottom;

	// rcMargins now contains the values used to shrink the printable
	// area of the page.

	// Convert device coordinates into logical coordinates
	DPtoLP(hdc, (LPPOINT) &rcMargins, 2);
	DPtoLP(hdc, (LPPOINT)&rcPhysMargins, 2);

	// Convert page size to logical units and we're done!
	DPtoLP(hdc, (LPPOINT) &ptPage, 1);

	DOCINFO di = {sizeof(DOCINFO), 0, 0, 0, 0};
	di.lpszDocName = m_strFileName;
	di.lpszOutput = 0;
	di.lpszDatatype = 0;
	di.fwType = 0;
	if(::StartDoc(hdc, &di)<0)
	{
		::MessageBox(m_hWnd,_T("Could not start the printer document."),_T("IEditor"),MB_OK|MB_ICONERROR);
		return 1;
	}

	LONG nLengthDoc = SendEditor(SCI_GETLENGTH,0,0);
	LONG nLengthDocMax = nLengthDoc;
	LONG nLengthPrinted = 0;

	// Requested to print selection
	if(pDlg.Flags & PD_SELECTION)
	{
		if(nStartPos > nEndPos)
		{
			nLengthPrinted = nEndPos;
			nLengthDoc = nStartPos;
		}
		else
		{
			nLengthPrinted = nStartPos;
			nLengthDoc = nEndPos;
		}

		if(nLengthPrinted < 0)	nLengthPrinted = 0;
		if(nLengthDoc > nLengthDocMax) nLengthDoc = nLengthDocMax;
	}

	// We must substract the physical margins from the printable area
	RangeToFormat frPrint;
	frPrint.hdc = hdc;
	frPrint.hdcTarget = hdc;
	frPrint.rc.left = rcMargins.left - rcPhysMargins.left;
	frPrint.rc.top = rcMargins.top - rcPhysMargins.top;
	frPrint.rc.right = ptPage.x - rcMargins.right - rcPhysMargins.left;
	frPrint.rc.bottom = ptPage.y - rcMargins.bottom - rcPhysMargins.top;
	frPrint.rcPage.left = 0;
	frPrint.rcPage.top = 0;
	frPrint.rcPage.right = ptPage.x - rcPhysMargins.left - rcPhysMargins.right - 1;
	frPrint.rcPage.bottom = ptPage.y - rcPhysMargins.top - rcPhysMargins.bottom - 1;
	// Print each page
	int nPageNum = 1;
	BOOL bPrintPage = FALSE;

	while(nLengthPrinted < nLengthDoc)
	{
		bPrintPage = (!(pDlg.Flags & PD_PAGENUMS) || (nPageNum >= pDlg.nFromPage) && (nPageNum <= pDlg.nToPage));
		if(bPrintPage) ::StartPage(hdc);

		frPrint.chrg.cpMin = nLengthPrinted;
		frPrint.chrg.cpMax = nLengthDoc;
		nLengthPrinted = SendEditor(SCI_FORMATRANGE, bPrintPage, reinterpret_cast<LPARAM>(&frPrint));
		
		if(bPrintPage) ::EndPage(hdc);
		nPageNum++;

		if((pDlg.Flags & PD_PAGENUMS) && (nPageNum > pDlg.nToPage)) break;
	}

	SendEditor(SCI_FORMATRANGE, FALSE, 0);

	::EndDoc(hdc);
	::DeleteDC(hdc);

	return 0;
}

LRESULT CLBandClass::OnEditUndo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	if(SendEditor(SCI_CANUNDO,0,0))	SendEditor(SCI_UNDO,0,0);
	return 0;
}

LRESULT CLBandClass::OnEditRedo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	if(SendEditor(SCI_CANREDO,0,0))	SendEditor(SCI_REDO,0,0);
	return 0;
}

LRESULT CLBandClass::OnEditCut(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SendEditor(SCI_CUT,0,0);
	return 0;
}

LRESULT CLBandClass::OnEditCopy(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SendEditor(SCI_COPY,0,0);
	return 0;
}

LRESULT CLBandClass::OnEditPaste(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SendEditor(SCI_PASTE,0,0);
	return 0;
}

LRESULT CLBandClass::OnEditSelectAll(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SendEditor(SCI_SETSEL,0,-1);
	return 0;
}

LRESULT CLBandClass::OnPageSource(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	PageSource();

	return 0;
}

// Mesage handler for find box.
LRESULT CALLBACK FindBox(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
		{
			CheckRadioButton(hDlg,IDC_RADIO_UP,IDC_RADIO_DOWN,IDC_RADIO_DOWN);
			::SendMessage(m_hwndFindTarget, SCI_GOTOPOS,0,0);
			return TRUE;
		}
		case WM_COMMAND:
		if(LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if(LOWORD(wParam) == IDOK)
		{
			int nFlags = 0;
			if(IsDlgButtonChecked(hDlg,IDC_CHECK_WHOLEWORD)) nFlags |= SCFIND_WHOLEWORD;
			if(IsDlgButtonChecked(hDlg,IDC_CHECK_MATCHCASE)) nFlags |= SCFIND_MATCHCASE;
			if(IsDlgButtonChecked(hDlg,IDC_CHECK_WRAP)) nFlags |= SCFIND_WORDSTART;
			if(IsDlgButtonChecked(hDlg,IDC_CHECK_REGEXP)) nFlags |= SCFIND_REGEXP;

			long nResult = -1;
			TCHAR szFindWhat[MAX_PATH];
			HWND hFindWhat = GetDlgItem(hDlg,IDC_EDIT_FINDWHAT);

			if(hFindWhat)
			{
				::GetWindowText(hFindWhat,szFindWhat,MAX_PATH-1);
				if(_tcslen(szFindWhat)>0)
				{
					if(m_hwndFindTarget)
					{
						long nTextLen = ::SendMessage(m_hwndFindTarget, SCI_GETTEXTLENGTH,0,0);
						if(m_nSearchAnchor<=0) m_nSearchAnchor = ::SendMessage(m_hwndFindTarget,SCI_GETCURRENTPOS,0,0);
						
						::SendMessage(m_hwndFindTarget,SCI_SETSELECTIONEND,::SendMessage(m_hwndFindTarget,SCI_GETSELECTIONSTART,0,0),0);

						TextToFind ttf = {{0, 0}, 0, {0, 0}};
#ifdef UNICODE
						char szValue[MAX_PATH];
						memset(szValue,0,MAX_PATH);
						WideCharToMultiByte(CP_ACP, 0, (USHORT*)szFindWhat, -1, szValue, MAX_PATH, NULL, NULL);
						ttf.lpstrText = const_cast<char*>(szValue);
#else
						ttf.lpstrText = const_cast<char*>(szFindWhat);
#endif
						ttf.chrg.cpMin = m_nSearchAnchor;
						if(IsDlgButtonChecked(hDlg,IDC_RADIO_DOWN)) ttf.chrg.cpMax = nTextLen;
						else ttf.chrg.cpMax = 0;
						ttf.chrgText.cpMin = 0;
						ttf.chrgText.cpMax = 0;

						nResult = ::SendMessage(m_hwndFindTarget, SCI_FINDTEXT, nFlags, reinterpret_cast<long>(&ttf));

						if(nResult>=0)
						{
							::SendMessage(m_hwndFindTarget, SCI_GOTOPOS,ttf.chrgText.cpMin,0);
							::SendMessage(m_hwndFindTarget, SCI_SETSEL, ttf.chrgText.cpMin,ttf.chrgText.cpMax);
							if(IsDlgButtonChecked(hDlg,IDC_RADIO_DOWN)) m_nSearchAnchor = ttf.chrgText.cpMax;
							else m_nSearchAnchor = ttf.chrgText.cpMin-1;
						}
						else m_nSearchAnchor = 0;
					}
				}
				
				if(nResult<0)
				{
					TCHAR szMessage[MAX_PATH+20];
					_tcscpy(szMessage,_T("Cannot find the string: "));
					_tcscat(szMessage,szFindWhat);
					::MessageBox(hDlg,szMessage,_T("IEditor"),MB_OK|MB_ICONINFORMATION);
				}
			}
			
			return TRUE;
		}
		break;
	}
    return FALSE;
}

LRESULT CLBandClass::OnEditFind(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	DialogBox(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDD_DIALOG_FIND), m_hwndScintilla, (DLGPROC)FindBox);
	return 0;
}

// Mesage handler for about box.
LRESULT CALLBACK AboutBox(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG: return TRUE;
		case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
    return FALSE;
}

LRESULT CLBandClass::OnHelpAbout(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	DialogBox(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDD_DIALOG_ABOUT), m_hwndScintilla, (DLGPROC)AboutBox);
	return 0;
}
