// LBandClass.h : Declaration of the CLBandClass
// *******************************************************************
// File    : LBandClass.h 
// Author  : Igor Tolmachev
// Email   : info@itsamples.com
// Web     : www.itsamples.com 
// You are free to use/modify this code but leave this header intact.
// *******************************************************************

#ifndef __LBANDCLASS_H_
#define __LBANDCLASS_H_

#include "resource.h"       // main symbols

static const char szElements[] = 
" a abbr acronym address applet area b base basefont "
" bdo big blockquote body br button caption center "
" cite code col colgroup dd del dfn dir div dl dt em "
" fieldset font form frame frameset h1 h2 h3 h4 h5 h6 "
" head hr html i iframe img input ins isindex kbd label "
" legend li link map menu meta noframes noscript "
" object ol optgroup option p param pre q s samp "
" script select small span strike strong style sub sup "
" table tbody td textarea tfoot th thead title tr tt u ul "
" var xml xmlns ";

static const char szAttributes[] =
" abbr accept-charset accept accesskey action align alink "
" alt archive axis background bgcolor border "
" cellpadding cellspacing char charoff charset checked cite "
" class classid clear codebase codetype color cols colspan "
" compact content coords "
" data datafld dataformatas datapagesize datasrc datetime "
" declare defer dir disabled enctype event "
" face for frame frameborder "
" headers height href hreflang hspace http-equiv "
" id ismap label lang language leftmargin link longdesc "
" marginwidth marginheight maxlength media method multiple "
" name nohref noresize noshade nowrap "
" object onblur onchange onclick ondblclick onfocus "
" onkeydown onkeypress onkeyup onload onmousedown "
" onmousemove onmouseover onmouseout onmouseup "
" onreset onselect onsubmit onunload "
" profile prompt readonly rel rev rows rowspan rules "
" scheme scope selected shape size span src standby start style "
" summary tabindex target text title topmargin type usemap "
" valign value valuetype version vlink vspace width "
" text password checkbox radio submit reset "
" file hidden image ";

/////////////////////////////////////////////////////////////////////////////
// CLBandClass
class ATL_NO_VTABLE CLBandClass : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CLBandClass, &CLSID_LBandClass>,
	public IDispatchImpl<ILBandClass, &IID_ILBandClass, &LIBID_IEDITORLib>,
	public CWindowImpl<CLBandClass>,
	public IObjectWithSiteImpl<CLBandClass>,
	public IDeskBand,
	public IPersistStreamInitImpl<CLBandClass>

{
public:
	CLBandClass()
	{
		m_pBrowser = NULL;
		m_pSite = NULL;
		m_hwndParent = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_LBANDCLASS)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CLBandClass)
	COM_INTERFACE_ENTRY(ILBandClass)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IObjectWithSite)
	COM_INTERFACE_ENTRY_IID(IID_IDeskBand,IDeskBand)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
END_COM_MAP()

BEGIN_PROP_MAP(CLBandClass)
END_PROP_MAP()

BEGIN_MSG_MAP(CLBandClass)
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_SIZE, OnSize)
	COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
	COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileOpen)
	COMMAND_ID_HANDLER(ID_FILE_SAVE, OnFileSave)
	COMMAND_ID_HANDLER(ID_FILE_SAVEAS, OnFileSaveAs)
	COMMAND_ID_HANDLER(ID_FILE_PRINT, OnFilePrint)
	COMMAND_ID_HANDLER(ID_EDIT_UNDO, OnEditUndo)
	COMMAND_ID_HANDLER(ID_EDIT_REDO, OnEditRedo)
	COMMAND_ID_HANDLER(ID_EDIT_CUT, OnEditCut)
	COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
	COMMAND_ID_HANDLER(ID_EDIT_PASTE, OnEditPaste)
	COMMAND_ID_HANDLER(ID_EDIT_SELECTALL, OnEditSelectAll)
	COMMAND_ID_HANDLER(ID_EDIT_FIND, OnEditFind)
	COMMAND_ID_HANDLER(ID_EDIT_SPECIAL, OnPageSource)
	COMMAND_ID_HANDLER(ID_HELP_ABOUT, OnHelpAbout)
END_MSG_MAP()

public:
	void FinalRelease() 
	{
		if(m_hScintilla) ::FreeLibrary(m_hScintilla);
	}

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

// ILBandClass
/////////////////////////////////////////////////////////////////////////////
// IObjectWithSite Methods
public:
	STDMETHODIMP SetSite(IUnknown* pUnkSite);
	STDMETHODIMP GetSite(REFIID riid, void **ppvSite);

/////////////////////////////////////////////////////////////////////////////
// IOleWindow Methods
public:
	STDMETHODIMP GetWindow(HWND* phwnd);
	STDMETHODIMP ContextSensitiveHelp(BOOL);

/////////////////////////////////////////////////////////////////////////////
// IDockingWindow, IDeskBand Methods
public:
	STDMETHODIMP ShowDW(BOOL fShow);
	STDMETHODIMP CloseDW(unsigned long dwReserved);
	STDMETHODIMP ResizeBorderDW(const RECT* prcBorder, IUnknown* punkToolbarSite, BOOL fReserved);
	STDMETHODIMP GetBandInfo(DWORD dwBandID, DWORD dwViewMode, DESKBANDINFO* pdbi);

//  Message Map
protected:
	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	
	LRESULT OnFileNew(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFileOpen(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFileSave(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFileSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFilePrint(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

	LRESULT OnEditUndo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditRedo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditCut(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditCopy(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditPaste(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditSelectAll(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditFind(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

	LRESULT OnPageSource(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

	LRESULT OnHelpAbout(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

protected:
	LRESULT SendEditor(UINT Msg, WPARAM wParam, LPARAM lParam);
	void SetAStyle(int style, COLORREF fore, COLORREF back, int size, const TCHAR* face);

	void FileControlInitialize(HWND hwnd);
	BOOL FileOpenDlg(HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName);
	BOOL FileSaveDlg(HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName);
	BOOL FileRead(PTSTR pstrFileName);
	BOOL FileWrite(PTSTR pstrFileName);
	BOOL ReadSettings();
	void PageSource();

public:
	BOOL        m_bRequiresSave;
	HWND        m_hwndScintilla;
	HINSTANCE   m_hScintilla;
	HGLOBAL     m_hDevMode;
	HGLOBAL     m_hDevNames;

private:
	IWebBrowser2*          m_pBrowser;
	IInputObjectSite*      m_pSite;
	HWND                   m_hwndParent;

protected:
	COLORREF m_crBackDefault;
	COLORREF m_crBackUnknown;
	COLORREF m_crTextDefault;
	COLORREF m_crTextSpecial;
	COLORREF m_crTextOther;
	COLORREF m_crTextComments;
	COLORREF m_crTextUnknown;
	COLORREF m_crTextTag;
	
	BOOL m_bKeywordBold;
	UINT m_nDefaultFontSize;
	UINT m_nHtmlFontSize;
	TCHAR m_strFontHtml[MAX_PATH];
	TCHAR m_strFontDefault[MAX_PATH];
	TCHAR m_strFileName[MAX_PATH];
	TCHAR m_strTitleName[MAX_PATH];
};

#endif //__LBANDCLASS_H_
