// ToolButton.cpp : Implementation of CToolButton
#include "stdafx.h"
#include "IEditor.h"
#include "ToolButton.h"

/////////////////////////////////////////////////////////////////////////////
// CToolButton

STDMETHODIMP CToolButton::QueryStatus(const GUID* pguidCmdGroup, ULONG cCmds, OLECMD prgCmds[], OLECMDTEXT* pCmdText)
{
	if (cCmds == 0) return E_INVALIDARG;
	if (prgCmds == 0) return E_POINTER;

	prgCmds[0].cmdf = OLECMDF_ENABLED;

	return S_OK;
}

STDMETHODIMP CToolButton::SetSite(IUnknown *pUnkSite)
{
	if(!pUnkSite)
	{
		ATLTRACE(_T("SetSite(): pUnkSite is NULL\n"));
	}
	else
	{
		HRESULT hRes = S_OK;
		CComPtr<IServiceProvider> spSP;
		hRes = pUnkSite->QueryInterface(&spSP);
		if(SUCCEEDED(hRes) && spSP)
			hRes = spSP->QueryService(IID_IWebBrowserApp, &m_pWebBrowser2);
	}
	
	m_spUnkSite = pUnkSite;
	m_bIsIe7 = FALSE;

	if(pUnkSite)
	{
		CComPtr<IDispatch>        pDocDisp;
		CComQIPtr<IHTMLDocument2> pHtmlDoc2;
		CComQIPtr<IHTMLWindow2>   pWindow;
		CComQIPtr<IOmNavigator>   pNavigator;

		HRESULT hRes = m_pWebBrowser2->get_Document(&pDocDisp);
		if(SUCCEEDED(hRes) && pDocDisp)
		{
			hRes = pDocDisp->QueryInterface(IID_IHTMLDocument2, (void**)&pHtmlDoc2);
			if(SUCCEEDED(hRes) && pHtmlDoc2)
			{
				hRes = pHtmlDoc2->get_parentWindow(&pWindow);
				if(SUCCEEDED(hRes) && pWindow)
				{
					hRes = pWindow->get_navigator(&pNavigator);
					if(SUCCEEDED(hRes) && pNavigator)
					{
						CComBSTR bstrVersion;
						hRes = pNavigator->get_appVersion(&bstrVersion);
						if(SUCCEEDED(hRes))
						{
							CString strVersion = CString(bstrVersion);
							if(strVersion.Find(_T("MSIE 7."))!=-1) m_bIsIe7 = TRUE;
						}
					}
				}
			}
		}
	}

	return S_OK;
}

STDMETHODIMP CToolButton::Exec(const GUID*, DWORD nCmdID, DWORD, VARIANTARG*, VARIANTARG*)
{
	if(m_spUnkSite == 0 || m_pWebBrowser2 == 0) return S_OK;

	CComVariant vtBandGUID;
	CComVariant vtShow;
	long nBrowser = 0;
	m_pWebBrowser2->get_HWND(&nBrowser);
	HWND hWndParent = (HWND)nBrowser;
	
	vtBandGUID.vt = VT_BSTR;
	vtBandGUID.bstrVal = SysAllocString(OLESTR("{50C36A9A-B65A-4D50-8122-0786231DDC28}"));
	vtShow.vt = VT_BOOL;
	vtShow.boolVal = !IsBarVisible(hWndParent);
	
	m_pWebBrowser2->ShowBrowserBar(&vtBandGUID, &vtShow, 0);
	
	SysFreeString(vtBandGUID.bstrVal);

	return S_OK;
}

BOOL CToolButton::IsBarVisible(HWND hWndBrowser)
{
	HWND hWnd = GetWindow(hWndBrowser, GW_CHILD);
	if(hWnd)
	{
		TCHAR szClassName[MAX_PATH];
		if(!m_bIsIe7)
		{
			while(hWnd)
			{
				memset(szClassName,0,MAX_PATH);
				GetClassName(hWnd, szClassName, MAX_PATH);
				if(_tcscmp(szClassName,_T("BaseBar"))==0)
				{
					while(hWnd)
					{
						hWnd = GetWindow(hWnd, GW_CHILD);
						GetClassName(hWnd, szClassName, MAX_PATH);
						if(_tcscmp(szClassName,_T("ReBarWindow32"))==0)
						{
							hWnd = GetWindow(hWnd, GW_CHILD);
							while(hWnd)
							{
								GetWindowText(hWnd, szClassName, MAX_PATH);
								if(_tcscmp(szClassName,_T("IEditorBarSuperClass"))==0)
								{
									return IsWindowVisible(hWnd);
								}
								hWnd = GetWindow(hWnd, GW_HWNDNEXT);
							}
						}
					}
				}
				hWnd = GetWindow(hWnd, GW_HWNDNEXT);
			}
		}
		else
		{
			while(hWnd)
			{
				memset(szClassName,0,MAX_PATH);
				GetClassName(hWnd, szClassName, MAX_PATH);
				if(_tcscmp(szClassName,_T("TabWindowClass"))==0)
				{
					if(IsWindowVisible(hWnd))
					{
						hWnd = GetWindow(hWnd, GW_CHILD);
						while(hWnd)
						{
							memset(szClassName,0,MAX_PATH);
							GetClassName(hWnd, szClassName, MAX_PATH);
							if(_tcscmp(szClassName,_T("BaseBar"))==0)
							{
								while(hWnd)
								{
									hWnd = GetWindow(hWnd, GW_CHILD);
									GetClassName(hWnd, szClassName, MAX_PATH);
									if(_tcscmp(szClassName,_T("ReBarWindow32"))==0)
									{
										hWnd = GetWindow(hWnd, GW_CHILD);
										while(hWnd)
										{
											GetWindowText(hWnd, szClassName, MAX_PATH);
											if(_tcscmp(szClassName,_T("IEditorBarSuperClass"))==0)
											{
												return IsWindowVisible(hWnd);
											}
											hWnd = GetWindow(hWnd, GW_HWNDNEXT);
										}
									}
								}
							}
							hWnd = GetWindow(hWnd, GW_HWNDNEXT);
						}
					}
				}
				hWnd = GetWindow(hWnd, GW_HWNDNEXT);
			}
		}
	}

	return FALSE;
}
